use mongodb::bson::doc;

use serenity::{
    async_trait,
    builder::CreateInteractionResponseData,
    model::{
        application::interaction::application_command::ApplicationCommandInteraction,
        prelude::{
            command::CommandOptionType, interaction::application_command::CommandDataOptionValue,
        },
    },
};

use crate::database::{models::Tag, Client as DatabaseClient};

use super::commands::{BotCommand, BotCommandOption};

struct CreateTagCommand {
    context: ApplicationCommandInteraction,
}

#[async_trait]
impl BotCommand for CreateTagCommand {
    fn name() -> String {
        String::from("create_tag")
    }

    fn description() -> String {
        String::from("Create a new tag")
    }

    fn options_list() -> Vec<BotCommandOption> {
        vec![BotCommandOption {
            name: String::from("tag"),
            description: String::from("The tag to create"),
            kind: CommandOptionType::String,
            required: true,
        }]
    }

    fn new(context: ApplicationCommandInteraction) -> Self {
        CreateTagCommand { context }
    }

    async fn run(&self, response: &mut CreateInteractionResponseData, database: &DatabaseClient) {
        let arg = self
            .context
            .data
            .options
            .get(0)
            .expect("Missing option")
            .resolved
            .as_ref()
            .expect("Could not deserialize option");

        let guild_id = self
            .context
            .guild_id
            .expect("Could not fetch guild id")
            .to_string();

        if let CommandDataOptionValue::String(tag_name) = arg {
            let matching_tags = database
                .get_all::<Tag>(Some(doc! {"name": tag_name, "guild_id": guild_id.as_str()}))
                .await;

            if !matching_tags.is_empty() {
                response.content("This tag already exist for this server");
            } else {
                match database
                    .insert_one(Tag {
                        id: None,
                        name: tag_name.to_string(),
                        guild_id: guild_id.clone(),
                        is_nsfw: false,
                        subscribers: vec![],
                    })
                    .await
                {
                    Ok(_) => response.content("Tag successfully created."),
                    Err(_) => response.content("Error creating the tag"),
                };
            }
        }
    }
}
